
/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/**************************************************************************
 *
 * cndiag.c:
 *
 * Broadcom cryptonet selftest diagnostics
 *
 **************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "ubsio.h"

#ifndef CRYPTONET_DEVICE
#define CRYPTONET_DEVICE "/dev/cryptonet"
#endif

#define VERMAJOR 0x01 
#define VERMINOR 0x04

const char *prog_name;

selftest(verbose,selftest,PCIDump, devicefail, param)
{
  int ubsec_fd;
  int retval;
  int lparam = param;
  int numcards;

  /*
   *  Open the encryption device
   */
  if ((ubsec_fd = open(CRYPTONET_DEVICE, O_RDWR, 0)) < 0) {
    printf("No CryptoNet device found\n");
    return(-1);
  }
  if (lparam > 0)
  {
     ioctl(ubsec_fd, UBSEC_GETNUMCARDS, &numcards);
     if (lparam > numcards)
     {
	if (numcards > 1)
           printf("Invalid card number specified to test. Card number should be between 1 and %d.\n",numcards);
	else
           printf("Invalid card number specified to test. Card number should be 1.\n");
	return -1;
     }
  }

  if (PCIDump) {
    retval=ioctl(ubsec_fd,UBSEC_DEVICEDUMP, &lparam);
  }
  else if (devicefail)
     retval=ioctl(ubsec_fd, UBSEC_FAILDEVICE, &lparam);
  else {
    printf("CryptoNet Device ");
    retval=ioctl(ubsec_fd,selftest ? UBSEC_SELFTEST : UBSEC_GETVERSION, &lparam);
    if (retval==-1) {
      if (selftest)
	printf("selftest failed.\n");
      else
	printf("not responding.\n");
      close(ubsec_fd);
      return(-1);
    }
    /*
    else if (retval > 0) {
      if (retval > 1)
        printf("Invalid card number specified to test. Card number should be between 1 and %d.\n", retval);
      else
        printf("Invalid card number specified to test. Card number should be 1.\n");
      return (-1);
    }
    */
    else
      if (selftest)
	printf("selftest passed.\n");
      else
	printf("installed.\n");
    
    if (verbose) {
      if (retval < 0)
         retval=-retval;
      printf("Device type BCM58%02x Revision %d. Driver version %x.%02x\n",
	     retval>>24,((retval>>16)&0xff),((retval>>8)&0xff),(retval&0xff));
    }
  }
  close(ubsec_fd);
}


int main(int argc, char *argv[])
{
  int Verbose=0;
  int Selftest=0;
  int PCIDump=0;
  int DeviceFail=0; 
  int Param = 0;
  int  c;
  char *tc = (char *) malloc(1);

  while ((c = getopt(argc, argv, "s::S::vVhHp::P::f::F::?xX")) != -1) {
    switch (c) {
    case 'v':
    case 'V':
      Verbose = 1;
      break;
    case 'p':
    case 'P':
      PCIDump = 1;
      if (optarg != NULL)
      {
	 Param = strtol(optarg, &tc, 10);
	 if (*tc != '\0')
	 {
	    printf("Invalid option parameter for option -%c\n", *tc);
	    exit(1);
	 }
      }
      printf("Unrecognized option %c\n", c);
      printf("Options: h(help) v(verbose) s<card number>(selftest) x(Diagnostic Version)\n");
      break;
    case 's':
    case 'S':
      Selftest = 1;
      if (optarg != NULL)
      {
	 Param = strtol(optarg, &tc, 10);
	 if (*tc != '\0')
	 {
	    printf("Invalid option parameter for option -%c\n", *tc);
	    exit(1);
	 }
      }
      break;
    case 'f':
    case 'F':
      DeviceFail = 1;
      if (optarg != NULL)
      {
	 Param = strtol(optarg, &tc, 10);
	 if (*tc != '\0')
	 {
	    printf("Invalid option parameter for option -%c\n", *tc);
	    exit(1);
	 }
      }
      printf("Unrecognized option %c\n", c);
      printf("Options: h(help) v(verbose) s<card number>(selftest) x(Diagnostic Version)\n");
      break;
    case '?':
    case 'h':
    case 'H':
      printf("Options: h(help) v(verbose) s<card number>(selftest) x(Diagnostic Version)\n");
      exit(0);
    case 'x':
    case 'X':
      printf("Diagnostic version %x.%02x\n", VERMAJOR, VERMINOR); 
      break;
    default:
      printf("Unrecognized option %c\n", c);
      exit(1);
    } 
  }

  exit(selftest(Verbose,Selftest,PCIDump, DeviceFail, Param));
}

