/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/**************************************************************************
 *
 *  File: file_io.c
 *
 **************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#ifndef USER_APPLICATION
#include "ubsio.h" /* Refer to wrapper for file I/O methods */
#endif

int Close(int fd)
{
  int status;
#ifdef USER_APPLICATION /* Separate user and kernel spaces build */
  if((status = close(fd)) == -1)
#else /* kernel build, use OS Wrapper mechanism for "file I/O" */
  if((status = OSW_DeviceClose(fd)) == -1)
#endif /* user/kernel build */
    fprintf(stderr, "Device close error\n");
  return status;
}



int Open(const char *pathname, int oflag, mode_t mode)
{
  int fd;
 
#ifdef USER_APPLICATION /* Separate user and kernel spaces build */
  if ( (fd = open(pathname, oflag, mode)) == -1) {
#else /* kernel build, use OS Wrapper mechanism for "file I/O" */
  if ( (fd = OSW_DeviceOpen(pathname, oflag, mode)) == -1) {
#endif /* user/kernel build */
    fprintf(stderr, "Unable to open \"%s\"\n", pathname);
  }
  return fd;
}





int Ioctl(int fd, long cmd, unsigned char *buf)
{
#ifdef USER_APPLICATION /* Separate user and kernel spaces build */
  return ioctl(fd, cmd, buf);
#else /* kernel build, use OS Wrapper mechanism for "ioctl" */
  return OSW_DeviceIoctl(fd, cmd, buf);
#endif /* user/kernel build */
}

