/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/******************************************************************************
 *
 *  Copyright 2001
 *  Broadcom Corporation
 *  16215 Alton Parkway
 *  PO Box 57013
 *  Irvine CA 92619-7013
 *
 *****************************************************************************/     

/*******************************************************************************
 *
 * File:         ubslib/ubsec_chipinfo_ioctl.c
 * 
 * What:         
 *
 * Description:
 *             
 * Revision History:
 *                   When       Who   What
 *                   03/13/01   DNA   Created
 *                   09/11/01   RJT   Added ubsec_chipinfo_ioctl function
 *                                    to replace ubsec_max_key_len_ioctl
 *                                    (functional superset)
 *
 ******************************************************************************/

#include <stdio.h>
#include <sys/ioctl.h>
#include "ubsec.h"
#include "ubsio.h"
#include "ubsec_lib.h"

/*
 * Global Variables:
 */

extern unsigned long user_timing;


/**************************************************************************
 *
 * Function:     ubsec_chipinfo_ioctl
 * 
 * Called From:  Any user application. 
 *               The chipinfo_structure must be allocated by the user app.
 *               The Status and CardNum members of this chipinfo_structure
 *               must be initialized by the user app prior to this call.
 *
 * What:         Returns driver and device (chip) information.
 * 
 * Description:  Fills the user allocated chipinfo_structure
 *               (ubsec_chipinfo_io_t defined in ../SRL/ubsec.h).
 *               This function is a superset of ubsec_max_key_len_ioctl(),
 *               and will eventually obsolete it.
 *
 * Return Values: 
 *                == 0   =>   Success
 *                != 0   =>   Failure
 *
 *************************************************************************/

int
ubsec_chipinfo_ioctl(
			int    fd,
			ubsec_chipinfo_io_t * chipinfo_structure
			) {

  if (chipinfo_structure->Status != sizeof(ubsec_chipinfo_io_t))
    return UBSEC_STATUS_NO_DEVICE;
  else 
    return ubsec_ioctl(fd, (unsigned char *)chipinfo_structure, UBSEC_EXTCHIPINFO_FUNC);
    
}


/**************************************************************************
 *
 * Function:     ubsec_max_key_len_ioctl
 * 
 * Called From:  Any user application.
 *
 * What:         Returns max key length in bits.
 * 
 * Description: 
 *
 * Return Values: 
 *                == 0   =>   Success
 *                != 0   =>   Failure
 *
 *************************************************************************/

int
ubsec_max_key_len_ioctl(
			int    fd,
			int *  max_key_len
			) {
  
  int                  status = 0;
  linux_chipinfo_io_t  cmd;
  
  do {
    
    bzero(&cmd, sizeof(linux_chipinfo_io_t));
    
    status = ubsec_ioctl(fd, (unsigned char *)&cmd, UBSEC_CHIPINFO_FUNC);
    
    if(status != 0) {
      status = -1;
      break;
    }
    
    *max_key_len = cmd.max_key_len;
    
    user_timing = cmd.time_us;
    
  } while(0);

  return(status);
}


