/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/******************************************************************************
 *
 *  Copyright 2000
 *  Broadcom Corporation
 *  16215 Alton Parkway
 *  PO Box 57013
 *  Irvine CA 92619-7013
 *
 *****************************************************************************/
/* 
 * Broadcom Corporation uBSec SDK 
 */
/*
 * ubsec_crypto.c:  Cryptographic functions.
 */
/*
 * Revision History:
 *
 * 05/24/00 JJT Created.
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include "ubsec.h"
#include "ubsio.h"
#include "ubsec_lib.h"

#define DEBUG_PRINTF

/*
 * Initialize crypto context.
 */
int ubsec_crypto_init(unsigned char *k1, unsigned char *k2, unsigned char *k3,
	unsigned char *auth_key, int encr_alg, int auth_alg, ubsec_crypto_context_p ctx)
{
	int	mac_length = 0;
	memset(ctx, 0, sizeof(*ctx));
	ctx->flags = encr_alg | auth_alg;
	if (UBSEC_USING_MAC(ctx->flags)) {
		mac_length = ((ctx->flags & MAC_MD5) ? MD5_LENGTH : SHA1_LENGTH);
	if (USING_MAC_STATES(ctx->flags)) 
		mac_length = sizeof(ubsec_HMAC_State_t) ; /* expect the states */

		memcpy(&ctx->auth[0], auth_key, mac_length); 
	}
	memcpy(&ctx->crypto[0], k1, 8); 
	memcpy(&ctx->crypto[8], k2, 8); 
	memcpy(&ctx->crypto[16], k3, 8); 
	return 0;
}

int ubsec_crypto_done(ubsec_crypto_context_p ctx)
{
	memset(ctx, 0, sizeof(ubsec_crypto_context_t));
	return 0;
}

int ubsec_crypto_data_ioctl(int fd, int cmd, ubsec_crypto_context_p ctx, 
	unsigned char *data_in, unsigned char *iv, unsigned short in_len,
	unsigned short crypto_offset, unsigned char *data_out, unsigned short out_len,
	unsigned long *optime)
{
	ubsec_io_t	at_buffer;
	ubsec_io_pt	at = &at_buffer;
	int		status = -1;
	unsigned short	mac_length = 0;
	int i;

	memset(at, 0, sizeof(ubsec_io_t));

	if (UBSEC_USING_MAC(ctx->flags)) {
	  mac_length = ((ctx->flags & MAC_MD5) ? MD5_LENGTH : SHA1_LENGTH);
	  if (USING_MAC_STATES(ctx->flags)) 
	    mac_length = sizeof(ubsec_HMAC_State_t); /* expect the states */
	  memcpy(at->mac_key, ctx->auth, mac_length); 

#if 0 /* Removed len check here pass through till wrapper header/auth checks do it at one place */
	  if (out_len < (in_len - mac_length)) {
#ifdef DEBUG_PRINTF
	    fprintf(stderr, "Error: output data too short (auth).\n");
#endif
	    return -1;
	  }
#endif
	} else {
#if 0 /* Removed len check here pass through till wrapper */
	  if (out_len < in_len) {
#ifdef DEBUG_PRINTF
	    fprintf(stderr, "Error: output data too short.\n");
#endif
	    return -1;
	  }
#endif
	}

	memcpy(at->crypt_key, ctx->crypto, MAX_CRYPT_KEY_LENGTH);
	memcpy(at->initial_vector, iv, IV_LENGTH);

	at->flags= ctx->flags | cmd;
	at->crypt_header_skip = crypto_offset;
	at->source_buf = data_in;
	at->source_buf_size = in_len;
	at->dest_buf = data_out;
	at->dest_buf_size = out_len;
	at->num_packets = 1;
	at->num_fragments = 1;
  
	status = ubsec_ioctl(fd, (unsigned char *)at, UBSEC_ENCRYPT_DECRYPT_FUNC);
	if (status) {
#ifdef DEBUG_PRINTF
		fprintf(stderr, "Ioctl Call Failed.\n");
#endif
		return status;
	}
	status = at->result_status;
	if (status) {
#ifdef DEBUG_PRINTF
		fprintf(stderr, "Ioctl Call Failed, Result Status=%d.\n", status);
#endif
		return status;
	}

#ifdef TIMING
#ifdef DEBUG_PRINTF
	if (at->time_us != 0) {
		fprintf(stderr, "Speed is %f mbps -- %luus\n", 
			(8 * input_size) / (double)time_us, time_us);
	}
#endif
#endif /* TIMING */

	if (optime) *optime = at->time_us;

	return 0;
}
	
   

