/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/******************************************************************************
 *
 *  Copyright 2000
 *  Broadcom Corporation
 *  16215 Alton Parkway
 *  PO Box 57013
 *  Irvine CA 92619-7013
 *
 *****************************************************************************/
/* 
 * Broadcom Corporation uBSec SDK 
 */
/*
 * ubsec_rng.c:  Random number generator library functions.
 */
/*
 * Revision History:
 *
 * 05/24/00 JJT Created.
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include "ubsec.h"
#include "ubsio.h"
#include "ubsec_lib.h"


extern unsigned long user_timing; 

/*
 * Random number function.
 */
int
rng_ioctl(int fd, ubsec_RNGCommand_t command,
	unsigned char *Result, int *Result_len)
{
	ubsec_rng_io_t		cmd;
	ubsec_RNGCommandParams_pt	params = &cmd.Rng;
	int			status = 0;

	bzero(&cmd, sizeof(cmd));
	cmd.command = command;

	/*
	 * Result will come back
	 */
	params->Result.KeyLength = *Result_len;
	params->Result.KeyValue = Result;

	/*
	 * Call chip.
	 */
	status = ubsec_ioctl (fd, (unsigned char *)&cmd, UBSEC_RNG_FUNC);

	/*
	 * Copy back results.
	 */
	if (status == UBSEC_STATUS_SUCCESS) {
		*Result_len = params->Result.KeyLength;
	}

	user_timing = cmd.time_us;

#ifdef TIMING
	if( user_timing != 0 )
		printf("Speed is %f mbps -- %luus\n", 
		(8 * input_size) / (double)user_timing, user_timing);
#endif /* TIMING */

	return status;
}


