/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/******************************************************************************
 *
 *  Copyright 2001
 *  Broadcom Corporation
 *  16215 Alton Parkway
 *  PO Box 57013
 *  Irvine CA 92619-7013
 *
 *****************************************************************************/     

/*******************************************************************************
 *
 * File:         ubslib/ubsec_stats.c
 * 
 * What:         
 *
 * Description:
 *              
 * Revision History:
 *                   When       Who   What
 *                   12/17/2001 DNA   Created
 *
 ******************************************************************************/

#include <stdio.h>
#include <sys/ioctl.h>
#include "ubsec.h"
#include "ubsio.h"
#include "ubsec_lib.h"

/*
 * Global Variables:
 */

extern unsigned long user_timing;


/**************************************************************************
 *
 * Function:    ubsec_stats_ioctl
 * 
 * Called From: Any user application.
 *
 * What:        
 * 
 * Description: 
 *              
 * Return Values: 
 *                == 0   =>   Success
 *                != 0   =>   Failure
 *
 *************************************************************************/

int
ubsec_stats_ioctl(
		  int            fd,
		  int            current_device,
		  ubsec_stats_pt stats
		  ) {
  
  int               status  = -1;
  ubsec_stats_io_t  cmd;
  
  bzero(&cmd, sizeof(ubsec_stats_io_t));

  cmd.device_num = current_device;

  status = ubsec_ioctl(fd, (unsigned char *)&cmd, UBSEC_STATS_FUNC);

  /*
   * We have two diff, though identical structs
   * one defined under the hood, the other which
   * users know about.
   */

  stats->BlocksEncryptedCount = cmd.dev_stats.BlocksEncryptedCount;
  stats->BlocksDecryptedCount = cmd.dev_stats.BlocksDecryptedCount;
  stats->BytesEncryptedCount  = cmd.dev_stats.BytesEncryptedCount;
  stats->BytesDecryptedCount  = cmd.dev_stats.BytesDecryptedCount;
  stats->CryptoFailedCount    = cmd.dev_stats.CryptoFailedCount;
  stats->IKECount             = cmd.dev_stats.IKECount;
  stats->IKEFailedCount       = cmd.dev_stats.IKEFailedCount;
  stats->DHPublicCount        = cmd.dev_stats.DHPublicCount;
  stats->DHSharedCount        = cmd.dev_stats.DHSharedCount;
  stats->RSAPublicCount       = cmd.dev_stats.RSAPublicCount;
  stats->RSAPrivateCount      = cmd.dev_stats.RSAPrivateCount;
  stats->DSASignCount         = cmd.dev_stats.DSASignCount;
  stats->DSAVerifyCount       = cmd.dev_stats.DSAVerifyCount;
  stats->DMAErrorCount        = cmd.dev_stats.DMAErrorCount;

  user_timing = cmd.time_us;

  return(status);
}

  
int
ubsec_numDevices_ioctl(int ubsec_fd,
		     int *numDevices) {
  
  return ioctl(ubsec_fd, UBSEC_GETNUMCARDS, numDevices);
  
}
