/*
 *  Broadcom Cryptonet Driver software is distributed as is, without any warranty
 *  of any kind, either express or implied as further specified in the GNU Public
 *  License. This software may be used and distributed according to the terms of
 *  the GNU Public License.
 *
 * Cryptonet is a registered trademark of Broadcom Corporation.
 */
/******************************************************************************
 *
 *  Copyright 2000
 *  Broadcom Corporation
 *  16215 Alton Parkway
 *  PO Box 57013
 *  Irvine CA 92619-7013
 *
 *****************************************************************************/
/* 
 * Broadcom Corporation uBSec SDK 
 */
/*
 * ubsec_utils.c:  Library device management utilities
 */
/*
 * Revision History:
 *
 * 12/10/99 SOR Created.
 */

#include <stdio.h>
#include <fcntl.h>
#include "ubsec.h"
#include "ubsio.h"
#include "ubsec_lib.h"
#include "file_io.h"



/*
 * Compute number of bits from number of bytes. Find the most significant bit
 * of the LongKey in the buffer.
 */
int
ubsec_bytes_to_bits(unsigned char *n, int bytes)
{
  UBS_UINT32    element;
  int		bits, ms_uint32;
  
  /* Make ms_uint32 index the uint32 element in the LongKey which might contain MSB */
  ms_uint32 = (bytes-1) >> 2;

  /* assume max # of MSbits for passed number of bytes (rounded up to uint32 boundary) */
  bits = ((bytes+3)&(~3)) * 8;

  while (!(element = ((UBS_UINT32 *)n)[ms_uint32])) {
    if (--ms_uint32 < 0)
      return 0; /* LongKey value is zero (no significant "1" bit) */ 
    bits -= 32;
  }
  
  while (!(element & 0x80000000)) {
    element <<= 1;
    bits --;
  }

  return bits;
}




/*
 * Compute required number of bytes needed to hold a LongKey of numbits bits.
 * Always returns an integer multiple of 4.
 */
int
ubsec_bits_to_bytes(int numbits)
{
	return ((((numbits + 7)>>3)+3)&(~3));
}





int
ubsec_open(char *device)
{
	int ubsec_fd;

#ifdef USE_USER_FILE
	if ((ubsec_fd = Open(device, O_RDWR, 0)) < 0) { 
#else
	if ((ubsec_fd = Open(UBSEC_KEY_DEVICE, O_RDWR, 0)) < 0) { 
#endif
#ifdef DEBUG
		fprintf(stderr, "ubsec_open: %s.\n", strerror(errno));
#endif /* DEBUG */
	} else {
		if (ubsec_fd == 0) {
#ifdef DEBUG
			fprintf(stderr, "ubsec_open failed.\n");
#endif /* DEBUG */
		}
	}
	return ubsec_fd;
}



int
ubsec_close(int fd)
{
  return Close(fd);
}







int
ubsec_ioctl (int ubsec_fd, unsigned char *buf, long cmd)
{
  int	status;
  
  status = Ioctl(ubsec_fd, cmd, buf);
  
  if (status  != UBSEC_STATUS_SUCCESS) {
#ifdef DEBUG
    fprintf(stderr, "Ioctl failed to ubsec device.\n");
#endif /* DEBUG */
  }
  return status;
}
